﻿XIncludeFile "OWire_Module.pbi"

UseModule OWire

EnableExplicit

Enumeration Window
  #WinMain
EndEnumeration

Enumeration Menu
  #Menu_Main
EndEnumeration

Enumeration Gadget
  #ComboPort
  #SpinThermoInterval
  #CheckThermoControl
  #ContainerThermoControl
  #SpinThermoMin
  #SpinThermoMax
  #Table
EndEnumeration

Enumeration
  #MItem_Log
  #MItem_Exit
  #MItem_Mode_CPU_MinUsage
  #MItem_Mode_Invert
  #MItem_ConvertOnly
  #MItem_Shem
  #MItem_About
EndEnumeration

Enumeration StatusBar
  #StatusBar_Main
EndEnumeration

Enumeration
  #StatusBar_Main_Status
EndEnumeration

Enumeration
  #Table_Type
  #Table_SN
  #Table_Power
  #Table_Thermo
  #Table_Status
EndEnumeration

Enumeration Image
  #ImageShem
EndEnumeration

Enumeration File
  #LogFileID
EndEnumeration

#Table_Color_Err  = $8888FD
#Table_Color_Warn = $ABF9FF

#DI_ErrFlag_CRC_ROM  = 1
#DI_ErrFlag_CRC_RAM  = 1<<1
#DI_ErrFlag_Thermo   = 1<<2
#DI_ErrFlag_OW       = 1<<3

CompilerIf #PB_Compiler_OS = #PB_OS_Windows
  #PortName = "COM"
CompilerElse
  #PortName = "/dev/ttyS"
CompilerEndIf

Structure ThreadParam
  BreakTh.a      ; Если #True, то нужно завершить работу потока.
  PortChange.a   ; Если #True, то выбран другой порт.
  ThermoControl.a
  ConvertOnly.a
  ThermoInterval.u ; Интервал в секундах опроса датчиков.
  ThermoMin.w
  ThermoMax.w
  Log.a
EndStructure

Structure DevicesInfo
  SN.OW_SN
  Thermo.f
  ErrFlag.a
EndStructure

Define Thread_OW_ID, Event, Window, x

Global gOW_Thread.ThreadParam
gOW_Thread\BreakTh    = #False
gOW_Thread\PortChange = #True

Procedure.s GetLastErrorString(Err)
  Protected String.s
  
  Select Err
    Case #OW_Err_OK       : String = "Нет ошибок"
    Case #OW_Err_Port     : String = "Нет открытого COM порта"
    Case #OW_Err_NoDevice : String = "Нет ответа от '1-Wire' устройств"
    Case #OW_Err_IO       : String = "Ошибка обмена данными"
    Case #OW_Err_BaudRate : String = "Ошибка изменения скорости порта"
    Case #OW_Err_Search   : String = "Ошибка при поиске '1-Wire' устройств"
    Case #OW_Err_BadParam : String = "Ошибка в аргументе функции"
    Case #OW_Err_NoData   : String = "Нет данных"
  EndSelect
  
  ProcedureReturn String
EndProcedure

Procedure WriteLog(String.s)
  If IsFile(#LogFileID)=0
    If OpenFile(#LogFileID, "DS18B20_ThermoMon.log")
      If Lof(#LogFileID)<=0
        WriteStringFormat(#LogFileID, #PB_UTF8)
      Else
        FileSeek(#LogFileID, Lof(#LogFileID))
      EndIf
      Goto _next
    EndIf
  Else
    _next:
    If String<>""
      ReplaceString(String, Chr($a), " ", #PB_String_InPlace)
      ReplaceString(String, Chr($d), " ", #PB_String_InPlace)
      String = FormatDate("[%dd/%mm/%yyyy %hh:%ii:%ss]  ", Date())+ String
    EndIf
    WriteStringN(#LogFileID, String, #PB_UTF8)
    FlushFileBuffers(#LogFileID)
  EndIf
  
  
EndProcedure

Procedure.s wordform(num.i, form_one.s, form_two.s, form_three.s) 
  num % 100 
  Protected num_two.i = num % 10 
  If num > 10 And num < 20 
    ProcedureReturn form_three 
  EndIf 
  If num_two > 1 And num_two < 5 
    ProcedureReturn form_two 
  EndIf 
  If num_two = 1
    ProcedureReturn form_one 
  EndIf 
  ProcedureReturn form_three 
EndProcedure

Procedure.s SN_Text(*x.OW_SN)
  Protected String.s, i
  
  String = RSet(Hex(*x\Type), 2, "0")+"-"
  
  For i=5 To 0 Step -1
    String + RSet(Hex(*x\Number[i]), 2, "0")
  Next i
  
  String + "-"+RSet(Hex(*x\CRC), 2, "0")
  
  ProcedureReturn String
EndProcedure

Procedure UpdateTableAll(List Devices.DevicesInfo())
  Protected String.s, i=0, Err
  
  ClearGadgetItems(#Table)
  ForEach Devices()
    Select Devices()\SN\Type
      Case $10
        String="DS1820 / DS18S20"
      Case $28
        String="DS18B20"
      Default
        String="Неизвестное устройство"
    EndSelect
    
    Err=#False
    ;Devices()\SN\crc+2
    String + Chr(10)+SN_Text(@Devices()\SN)
    If OW_GetCRC(Devices()\SN, 7) <> Devices()\SN\CRC
      String+"  ОШИБКА CRC" : Err=#True
      Devices()\ErrFlag | #DI_ErrFlag_CRC_ROM
    EndIf
    
    String+Chr(10)
    
    If OW_SelectROM(Devices()\SN)
      If OW_ReadPowerSupply()
        String+"3 вывода"
      Else
        String+"2 вывода ('паразитное' питание)"
      EndIf
    Else
      String+"Ошибка"
    EndIf
    
    If Devices()\SN\Type=$10 Or Devices()\SN\Type=$28
      String+Chr(10)+"Измерение"+Chr(10)+"Инициализация"
    EndIf
    AddGadgetItem(#Table, i, String)
    If Err=#True
      SetGadgetItemColor(#Table, i, #PB_Gadget_BackColor, #Table_Color_Err, #Table_SN)
    EndIf
    i+1
  Next
  
EndProcedure

Procedure SearchSensor(List Devices.DevicesInfo(), Port.s)
  Protected Sensor.OW_SN, x=0
  
  StatusBarText(#StatusBar_Main, #StatusBar_Main_Status, "Поиск '1-Wire' устройств...")
    
  If ListSize(Devices())>0
    ClearList(Devices())
  EndIf
    
  OW_SearchROM_Reset()
  
  
  While OW_SearchROM_Enum(@Sensor)=#True
    x+1
    If AddElement(Devices())
      CopyStructure(@Sensor, @Devices()\SN, OW_SN)
    EndIf
  Wend
    
  If x>0 
    UpdateTableAll(Devices())
    StatusBarText(#StatusBar_Main, #StatusBar_Main_Status,
                  "Найдено "+x+" '1-Wire' "+wordform(x, "устройство", " устройства", " устройств")+".")
  Else
    ClearGadgetItems(#Table)
    StatusBarText(#StatusBar_Main, #StatusBar_Main_Status,
                  "На порту "+Port+", '1-Wire' устройства не найдены.")
  EndIf
EndProcedure

Procedure SelectPort(List Devices.DevicesInfo())
  Protected Port.s
  
  Port = GetGadgetText(#ComboPort)
  
  OW_ClosePort()
  
  If Port<>""
    If OW_OpenPort(Port)
      StatusBarText(#StatusBar_Main, #StatusBar_Main_Status, "Порт открыт")
    Else
      StatusBarText(#StatusBar_Main, #StatusBar_Main_Status,
                    "Не удалось открыть порт '"+Port+
                    "'  Возможно он зянят или отстуствует в системе")
    EndIf
  EndIf
  
  ClearList(Devices())
  
  If OW_GetLastError() = #OW_Err_OK        ; Ошибок нет.
    SearchSensor(Devices(), Port)
  EndIf
  
EndProcedure


Procedure Thread_OW(*Bool)
  Protected NewList Devices.DevicesInfo()
  Protected String.s, Count, i, Err, OldPos, z
  Protected x.w, Thermo.f, RAM.OW_RAM
  Protected CountErr, OldCountErr
  Protected WarnCount, OldWarnCount
  
  Repeat
    
    Delay(100)
    
    If gOW_Thread\PortChange
      SelectPort(Devices())
      Count = ListSize(Devices())
      If Count>0
        ResetList(Devices())
        i=0
        OldPos=0
      EndIf
      gOW_Thread\PortChange = #False
    EndIf
    
    If Count>0
      
      Repeat
        
        If NextElement(Devices())=0
          i=0
          If gOW_Thread\ConvertOnly=0 ; Раздельный опрос отключен
            ResetList(Devices())
            Break
          Else                        ; Раздельный опрос включен
            If FirstElement(Devices())=0
              Break
            EndIf
          EndIf
        EndIf
        
        
        
        If gOW_Thread\ConvertOnly=0 ; Раздельный опрос отключен
          
          If i=0
            If OW_SkipROM()=#False
              String = GetLastErrorString(OW_GetLastError())
              StatusBarText(#StatusBar_Main, #StatusBar_Main_Status, String)
              Err=#True
              Goto M1
            EndIf
          EndIf
          
        Else                        ; Раздельный опрос включен
          
          If OW_SelectROM(@Devices()\SN)=#False
            String = GetLastErrorString(OW_GetLastError())
            StatusBarText(#StatusBar_Main, #StatusBar_Main_Status, String)
            Err=#True
            Goto M1
          EndIf
          
        EndIf
        
        If i=0 Or gOW_Thread\ConvertOnly
          If OW_ConvertT()=#False
            String = GetLastErrorString(OW_GetLastError())
            StatusBarText(#StatusBar_Main, #StatusBar_Main_Status, String)
            Err=#True
            Goto M1
          EndIf
          
          CountErr=0
          WarnCount=0
          
          
          For z=1 To 8 ; Ждем окончания измерения температуры.
            Delay(100)
            If gOW_Thread\PortChange
              Break
            ElseIf gOW_Thread\BreakTh
              Break 2
            EndIf
          Next z
          
        EndIf
        
        
        
        If Devices()\SN\Type = $10 Or ; DS1820 / DS18S20
           Devices()\SN\Type = $28    ; DS18B20
          
          If OW_SelectROM(@Devices()\SN)
            If OW_ReadRAM(@RAM)
              x=(RAM\H_Termo<<8)|RAM\L_Termo
              If Devices()\SN\Type = $28 ; DS18B20
                Thermo=x/16
              Else
                Thermo=x/2
              EndIf
              Devices()\Thermo = Thermo
              SetGadgetItemText(#Table, i, StrF(Thermo, 1)+" °C", #Table_Thermo)
              
              If gOW_Thread\Log
                WriteLog(SN_Text(@Devices()\SN)+"  —  "+StrF(Thermo, 1)+" °C")
              EndIf
              
              
              If Thermo<-55 Or Thermo>127
                Devices()\ErrFlag | #DI_ErrFlag_Thermo
                SetGadgetItemColor(#Table, i, #PB_Gadget_BackColor, #Table_Color_Warn, #Table_Thermo)
              Else
                Devices()\ErrFlag & ~#DI_ErrFlag_Thermo
                x/16
                If gOW_Thread\ThermoControl=0 Or
                   (gOW_Thread\ThermoMin<=x And x<=gOW_Thread\ThermoMax)
                  SetGadgetItemColor(#Table, i, #PB_Gadget_BackColor, -1, #Table_Thermo)
                Else
                  WarnCount+1
                  SetGadgetItemColor(#Table, i, #PB_Gadget_BackColor, #Table_Color_Warn, #Table_Thermo)
                EndIf
              EndIf
              If OW_GetCRC(@RAM, 8)=RAM\CRC
                Devices()\ErrFlag & ~#DI_ErrFlag_CRC_RAM
                SetGadgetItemText(#Table, i, "OK", #Table_Status)
                SetGadgetItemColor(#Table, i, #PB_Gadget_BackColor, -1, #Table_Status)
              Else
                Devices()\ErrFlag | #DI_ErrFlag_CRC_RAM
                SetGadgetItemText(#Table, i, "Ошибка CRC", #Table_Status)
                SetGadgetItemColor(#Table, i, #PB_Gadget_BackColor, #Table_Color_Err, #Table_Status)
              EndIf
            EndIf
          EndIf
          
          If OW_GetLastError()<>#OW_Err_OK 
            Devices()\ErrFlag | #DI_ErrFlag_OW
            SetGadgetItemText(#Table, i, GetLastErrorString(OW_GetLastError()), #Table_Status)
            SetGadgetItemColor(#Table, i, #PB_Gadget_BackColor, #Table_Color_Err, #Table_Status)
          ElseIf Devices()\ErrFlag & #DI_ErrFlag_CRC_RAM = 0
            Devices()\ErrFlag & ~#DI_ErrFlag_OW
            SetGadgetItemColor(#Table, i, #PB_Gadget_BackColor, -1, #Table_Status)
          EndIf
          
        Else
          Continue
        EndIf
        
        
        
        If Devices()\ErrFlag
          CountErr+1
          If Devices()\ErrFlag & ~#DI_ErrFlag_CRC_ROM = 0
            SetGadgetItemText(#Table, i, "", #Table_Thermo)
          EndIf
        EndIf
        
        If gOW_Thread\ConvertOnly ; Раздельный опрос включен
          SetGadgetItemColor(#table, OldPos, #PB_Gadget_BackColor, -1, #Table_Type)
          SetGadgetItemColor(#table, i, #PB_Gadget_BackColor, RGB(128, 255, 128), #Table_Type)
        EndIf
        
        OldPos=i
        i+1
        
        If gOW_Thread\ConvertOnly ; Раздельный опрос включен
          Break
        EndIf
        
      ForEver
      
      
      If Err=#True Or CountErr<>OldCountErr Or OldWarnCount<>WarnCount
        
        If CountErr>0
          String=".    "+wordform(CountErr, "Обнаружен ", "Обнаружено ", "Обнаружены ")+
                 CountErr+wordform(CountErr, " датчик", " датчика", " датчиков")+
                 " с ошибками в работе"
        Else
          String="."
        EndIf
        
        If WarnCount>0
          String+"  В "+WarnCount+wordform(WarnCount, " датчике", " датчиках", " датчиках")+
                 " зафиксирован выход за пределы допустимой температуры"
        EndIf
        
        StatusBarText(#StatusBar_Main, #StatusBar_Main_Status,
                      "Найдено "+Count+" '1-Wire' "+
                      wordform(Count, "устройство", " устройства", " устройств")+String)
        Err=#False
        OldCountErr = CountErr
        OldWarnCount=WarnCount
      EndIf
      
      
      If gOW_Thread\ThermoInterval>1
        For z=1 To gOW_Thread\ThermoInterval-1
          For x=1 To 10
            Delay(100)
            If gOW_Thread\PortChange Or gOW_Thread\BreakTh
              Break 2
            EndIf
          Next x
        Next z
      EndIf
     
      
      
    Else
      
      M1:
      If WarnCount<>-1
        x=CountGadgetItems(#Table)-1
        For i=0 To x
          SetGadgetItemText(#Table, i, "", #Table_Thermo)
          SetGadgetItemText(#Table, i, String, #Table_Status)
          SetGadgetItemColor(#Table, i, #PB_Gadget_BackColor, -1, #Table_Thermo)
          SetGadgetItemColor(#Table, i, #PB_Gadget_BackColor, #Table_Color_Err, #Table_Status)
        Next
        WarnCount=-1
      EndIf
      
      CountErr+1
      If CountErr>=10
        Count=0
        If OW_Reset()
          SearchSensor(Devices(), GetGadgetText(#ComboPort))
          Count = ListSize(Devices())
        EndIf
        CountErr=0
      EndIf
      
      
    EndIf
    
  Until gOW_Thread\BreakTh
  
EndProcedure

Procedure EnumPorts(Gadget)
  Protected i, ID
  
  ClearGadgetItems(Gadget)
  For i=0 To 10
    ID = OpenSerialPort(#PB_Any, #PortName+i, 9600, #PB_SerialPort_NoParity,
                      8, 1, #PB_SerialPort_NoHandshake, 255, 255)
    If ID
      AddGadgetItem(Gadget, -1, #PortName+i)
      CloseSerialPort(ID)
    EndIf
  Next
EndProcedure

Procedure OpenWinMain()
  Protected x
  
  CompilerIf #PB_Compiler_OS = #PB_OS_Linux
    SetGadgetFont(#PB_Default, FontID(LoadFont(#PB_Any,"Liberation Mono",10)))
  CompilerEndIf
  
  OpenWindow(#WinMain, 0, 0, 590, 400, "DS18B20 термо-мониторинг v1.1", 
             #PB_Window_MinimizeGadget|#PB_Window_Invisible|#PB_Window_ScreenCentered)
  
  CreateMenu(#Menu_Main, WindowID(#WinMain))
  MenuTitle("Файл")
    MenuItem(#MItem_Log, "Записывать в лог-файл")
    MenuBar()
    MenuItem(#MItem_Exit, "Выход")
  MenuTitle("Режим")
    MenuItem(#MItem_Mode_CPU_MinUsage, "Снизить нагрузку на CPU")
    MenuItem(#MItem_Mode_Invert, "Инвертировать DTR и RTS")
    MenuItem(#MItem_ConvertOnly, "Раздельный опрос")
  MenuTitle("Справка")
    MenuItem(#MItem_About, "О программе")
  
  x=TextGadget(#PB_Any, 4, 8, 100, 16, "Порт:")
  ResizeGadget(x, #PB_Ignore, #PB_Ignore, GadgetWidth(x, #PB_Gadget_RequiredSize)+4, #PB_Ignore)
  
  ComboBoxGadget(#ComboPort, GadgetX(x)+GadgetWidth(x), 4, 100, 22)
  EnumPorts(#ComboPort)
  
  
  x=TextGadget(#PB_Any, GadgetX(#ComboPort)+GadgetWidth(#ComboPort)+10, 8, 350, 16, "Интервал опроса:")
  ResizeGadget(x, #PB_Ignore, #PB_Ignore, GadgetWidth(x, #PB_Gadget_RequiredSize)+4, #PB_Ignore)
  
  SpinGadget(#SpinThermoInterval, GadgetX(x)+GadgetWidth(x), 4, 50, 22, 1, 60,
             #PB_Spin_ReadOnly|#PB_Spin_Numeric)
  GadgetToolTip(#SpinThermoInterval, "Интервал в секундах опроса датчиков")
  
  CheckBoxGadget(#CheckThermoControl, GadgetX(#SpinThermoInterval)+GadgetWidth(#SpinThermoInterval)+16,
                 8, 100, 16, "Термо контроль")
  ResizeGadget(#CheckThermoControl, #PB_Ignore, #PB_Ignore,
               GadgetWidth(#CheckThermoControl, #PB_Gadget_RequiredSize)+2, #PB_Ignore)
  
;   CheckBoxGadget(#CheckConvertOnly, GadgetX(#CheckThermoControl),
;                  GadgetY(#CheckThermoControl)+GadgetHeight(#CheckThermoControl)+2, 100, 16, )
;   ResizeGadget(#CheckConvertOnly, #PB_Ignore, #PB_Ignore,
;                GadgetWidth(#CheckConvertOnly, #PB_Gadget_RequiredSize)+2, #PB_Ignore)
;   GadgetToolTip(#CheckConvertOnly, "Опрос датчиков по одному. Необходимо для работы с множеством датчиков с 'паразитным' питанием. Внимание! Скорость считывания уменьшается по мере увеличения количества датчиков")
  
  ContainerGadget(#ContainerThermoControl, GadgetX(#CheckThermoControl)+GadgetWidth(#CheckThermoControl),
                  2, 200, 28, #PB_Container_Single)
  
  x=TextGadget(#PB_Any, 4, 7, 350, 16, "Мин:")
  ResizeGadget(x, #PB_Ignore, #PB_Ignore, GadgetWidth(x, #PB_Gadget_RequiredSize)+4, #PB_Ignore)
  
  SpinGadget(#SpinThermoMin, GadgetX(x)+GadgetWidth(x), 2, 50, 22, -55, 125,
             #PB_Spin_ReadOnly|#PB_Spin_Numeric)
  GadgetToolTip(#SpinThermoMin, "Минимальная допустимая температура")
  
  x=TextGadget(#PB_Any, GadgetX(#SpinThermoMin)+GadgetWidth(#SpinThermoMin)+10, 7, 350, 16, "Макс:")
  ResizeGadget(x, #PB_Ignore, #PB_Ignore, GadgetWidth(x, #PB_Gadget_RequiredSize)+4, #PB_Ignore)
  
  SpinGadget(#SpinThermoMax, GadgetX(x)+GadgetWidth(x), 2, 50, 22, -55, 125,
             #PB_Spin_ReadOnly|#PB_Spin_Numeric)
  GadgetToolTip(#SpinThermoMax, "Максимальная допустимая температура")
  
  CloseGadgetList()
  
  ResizeGadget(#ContainerThermoControl, #PB_Ignore, #PB_Ignore, 
               GadgetX(#SpinThermoMax)+GadgetWidth(#SpinThermoMax)+10, #PB_Ignore)
  
  ResizeWindow(#WinMain, #PB_Ignore, #PB_Ignore, 
               GadgetX(#ContainerThermoControl)+GadgetWidth(#ContainerThermoControl)+2, #PB_Ignore)
  
  x=ListIconGadget(#Table, 2, GadgetY(#ContainerThermoControl)+GadgetHeight(#ContainerThermoControl)+2, 
                   WindowWidth(#WinMain)-4, 304, "Тип", 80, 
                   #PB_ListIcon_GridLines|#PB_ListIcon_FullRowSelect|#PB_ListIcon_AlwaysShowSelection)
  
  CompilerIf #PB_Compiler_OS = #PB_OS_Windows
    SendMessage_(x, #LVM_SETEXTENDEDLISTVIEWSTYLE, 
                 #LVS_EX_LABELTIP|#LVS_EX_GRIDLINES|#LVS_EX_DOUBLEBUFFER, 
                 #LVS_EX_LABELTIP|#LVS_EX_GRIDLINES|#LVS_EX_DOUBLEBUFFER)
  CompilerEndIf
  
  AddGadgetColumn(#Table, #Table_SN, "Серийный номер", 150)
  AddGadgetColumn(#Table, #Table_Power, "Подключение", 100)
  AddGadgetColumn(#Table, #Table_Thermo, "Температура", 100)
  AddGadgetColumn(#Table, #Table_Status, "Статус", 120)
  
  CreateStatusBar(#StatusBar_Main, WindowID(#WinMain))
  AddStatusBarField(#PB_Ignore)
  
  ResizeGadget(#Table, #PB_Ignore, #PB_Ignore, #PB_Ignore,
               WindowHeight(#WinMain)-StatusBarHeight(#StatusBar_Main)-GadgetY(#Table)-MenuHeight()-2)
  
  HideWindow(#WinMain, #False)
EndProcedure

Procedure ReadSetting()
  Protected String.s, x,Flag, p 
  
  x=OpenPreferences("ThermoMon.ini")
  
  String=ReadPreferenceString("Port", "")
  If String
    SetGadgetText(#ComboPort, String)
  Else
    SetGadgetState(#ComboPort, -1)
  EndIf
  gOW_Thread\ThermoInterval = ReadPreferenceLong("Interval", 1)
  If gOW_Thread\ThermoInterval<=0 Or gOW_Thread\ThermoInterval>100
    gOW_Thread\ThermoInterval=1
  EndIf
  SetGadgetState(#SpinThermoInterval, gOW_Thread\ThermoInterval)
  SetGadgetText(#SpinThermoInterval, Str(gOW_Thread\ThermoInterval))
  
  gOW_Thread\ThermoControl = ReadPreferenceLong("ThermoControl", 0)&1
  SetGadgetState(#CheckThermoControl, gOW_Thread\ThermoControl)
  
  gOW_Thread\ConvertOnly = ReadPreferenceLong("ConvertOnly", 0)&1
  SetMenuItemState(#Menu_Main, #MItem_ConvertOnly, gOW_Thread\ConvertOnly)
  
  gOW_Thread\ThermoMin = ReadPreferenceLong("ThermoMin", 0)
  gOW_Thread\ThermoMax = ReadPreferenceLong("ThermoMax", 10)
  
  gOW_Thread\Log = ReadPreferenceLong("Log", 0)&1
  SetMenuItemState(#Menu_Main, #MItem_Log, gOW_Thread\Log)
  
  Flag = OW_GetMode()
  p = ReadPreferenceLong("CPU_MinUsage", 0)&1
  If p
    Flag|#OW_Mode_CPU_MinUsage
  EndIf
  SetMenuItemState(#Menu_Main, #MItem_Mode_CPU_MinUsage, p)
  
  p=ReadPreferenceLong("Invert", 0)
  If p
    Flag|#OW_Mode_Invert
  EndIf
  SetMenuItemState(#Menu_Main, #MItem_Mode_Invert, p)
  
  OW_SetMode(Flag)                 
  
  If gOW_Thread\ThermoMin<=-55 Or gOW_Thread\ThermoMin>130
    gOW_Thread\ThermoMin=0
  EndIf
  
  If gOW_Thread\ThermoMax<=-55 Or gOW_Thread\ThermoMax>130
    gOW_Thread\ThermoMax=10
  EndIf
  
  If gOW_Thread\ThermoMin>=gOW_Thread\ThermoMax
    gOW_Thread\ThermoMin=gOW_Thread\ThermoMax-1
  EndIf
  
  SetGadgetState(#SpinThermoMin, gOW_Thread\ThermoMin)
  SetGadgetText(#SpinThermoMin, Str(gOW_Thread\ThermoMin))
   
  
  SetGadgetState(#SpinThermoMax, gOW_Thread\ThermoMax)
  SetGadgetText(#SpinThermoMax, Str(gOW_Thread\ThermoMax))
  
  PostEvent(#PB_Event_Gadget, #WinMain, #ComboPort, #PB_EventType_Change)
  PostEvent(#PB_Event_Gadget, #WinMain, #CheckThermoControl)
  
  If x
    ClosePreferences()
  EndIf
EndProcedure

Procedure WriteSetting()
  If CreatePreferences("ThermoMon.ini")
    
    WritePreferenceString("Port", GetGadgetText(#ComboPort))
    WritePreferenceLong("Interval", GetGadgetState(#SpinThermoInterval))
    WritePreferenceLong("ThermoControl", GetGadgetState(#CheckThermoControl))
    WritePreferenceLong("ConvertOnly", GetMenuItemState(#Menu_Main, #MItem_ConvertOnly))
    WritePreferenceLong("ThermoMin", GetGadgetState(#SpinThermoMin))
    WritePreferenceLong("ThermoMax", GetGadgetState(#SpinThermoMax))
    WritePreferenceLong("Log", gOW_Thread\Log)
    WritePreferenceLong("CPU_MinUsage", GetMenuItemState(#Menu_Main, #MItem_Mode_CPU_MinUsage))
    WritePreferenceLong("Invert", GetMenuItemState(#Menu_Main, #MItem_Mode_Invert))
    
    ClosePreferences()
  EndIf
EndProcedure


Procedure PortChange()
  gOW_Thread\PortChange = #True
  WriteSetting()
EndProcedure

SetCurrentDirectory(GetPathPart(ProgramFilename()))

OpenWinMain()
BindGadgetEvent(#ComboPort, @PortChange(), #PB_EventType_Change)

ReadSetting()

Thread_OW_ID=CreateThread(@Thread_OW(), 0)

Repeat
  Event = WaitWindowEvent()
  Window = EventWindow()
  
  If Window = #WinMain
    If Event = #PB_Event_Menu
      Select EventMenu()
        Case #MItem_Log
          gOW_Thread\Log = GetMenuItemState(#Menu_Main, #MItem_Log)!1
          SetMenuItemState(#Menu_Main, #MItem_Log, gOW_Thread\Log)
          WriteSetting()
        Case #MItem_Exit : Break
        Case #MItem_Mode_CPU_MinUsage
          x=GetMenuItemState(#Menu_Main, #MItem_Mode_CPU_MinUsage)!1
          SetMenuItemState(#Menu_Main, #MItem_Mode_CPU_MinUsage, x)
          If x
            OW_SetMode(OW_GetMode()|#OW_Mode_CPU_MinUsage)
          Else
            OW_SetMode(OW_GetMode()&~#OW_Mode_CPU_MinUsage)
          EndIf
        Case #MItem_Mode_Invert
          x=GetMenuItemState(#Menu_Main, #MItem_Mode_Invert)!1
          SetMenuItemState(#Menu_Main, #MItem_Mode_Invert, x)
          If x
            OW_SetMode(OW_GetMode()|#OW_Mode_Invert)
          Else
            OW_SetMode(OW_GetMode()&~#OW_Mode_Invert)
          EndIf
        Case #MItem_ConvertOnly
          x = GetMenuItemState(#Menu_Main, #MItem_ConvertOnly) ! 1
          gOW_Thread\ConvertOnly = x
          SetMenuItemState(#Menu_Main, #MItem_ConvertOnly, x)
          If x=0
            SetGadgetItemColor(#table, -1, #PB_Gadget_BackColor, -1, #Table_Type)
          EndIf
        Case #MItem_About
          MessageRequester("О программе", "Мониторинг температуры с использованием DS18x20."+Chr(10)+
                                          "E-Mail: pure-basic@yandex.ru")
      EndSelect
      
    ElseIf Event = #PB_Event_Gadget
      Select EventGadget()
        Case #SpinThermoInterval
          gOW_Thread\ThermoInterval = GetGadgetState(#SpinThermoInterval)
          WriteSetting()
        Case #CheckThermoControl
          gOW_Thread\ThermoControl = GetGadgetState(#CheckThermoControl)
          WriteSetting()          
        Case #SpinThermoMin
          x=GetGadgetState(#SpinThermoMin)
          Select EventType()
            Case #PB_EventType_Change
              gOW_Thread\ThermoMin = x
              WriteSetting()
            Case #PB_EventType_Up
              If x>=GetGadgetState(#SpinThermoMax)
                x=GetGadgetState(#SpinThermoMax)-1
                SetGadgetState(#SpinThermoMin, x)
                SetGadgetText(#SpinThermoMin, Str(x))
              EndIf
          EndSelect
        Case #SpinThermoMax
          x=GetGadgetState(#SpinThermoMax)
          Select EventType()
            Case #PB_EventType_Change
              gOW_Thread\ThermoMax = x
              WriteSetting()
            Case #PB_EventType_Down
              If x=<GetGadgetState(#SpinThermoMin)
                x=GetGadgetState(#SpinThermoMin)+1
                SetGadgetState(#SpinThermoMax, x)
                SetGadgetText(#SpinThermoMax, Str(x))
              EndIf
          EndSelect
      EndSelect
    EndIf
  EndIf
  
Until Event = #PB_Event_CloseWindow And Window = #WinMain

gOW_Thread\BreakTh=#True
HideWindow(#WinMain, #True)
WriteSetting()
WaitThread(Thread_OW_ID, 400)
If IsThread(Thread_OW_ID)
  KillThread(Thread_OW_ID)
EndIf

If IsFile(#LogFileID)
  CloseFile(#LogFileID)
EndIf

End

; IDE Options = PureBasic 5.24 LTS (Windows - x86)
; CursorPosition = 265
; FirstLine = 254
; Folding = ---
; EnableUnicode
; EnableThread
; EnableXP
; UseIcon = thermometer.ico
; Executable = ..\ThermoMon.exe
; CompileSourceDirectory
; IncludeVersionInfo
; VersionField0 = 1.1.0.0
; VersionField1 = 1.1.0.0
; VersionField2 = ϸ��
; VersionField3 = DS18B20 ThermoMon
; VersionField4 = 1.1
; VersionField5 = 1.1
; VersionField6 = DS18B20 �����-����������
; VersionField7 = DS18B20 ThermoMon
; VersionField8 = %EXECUTABLE
; VersionField13 = pure-basic@yandex.ru
; VersionField15 = VOS_NT_WINDOWS32
; VersionField16 = VFT_APP
; VersionField17 = 0419 Russian